@@ -13,7 +13,6 @@ import logging
 import os
 import re
 import subprocess
-import sys
 import time
 import threading
 import traceback
@@ -405,8 +404,6 @@ class ESProcess(threading.Thread):
         msg = None
         if "No more events" in event_ranges:
             msg = event_ranges
-            if (sys.version_info > (3, 0)):  # needed for Python 3
-                msg = msg.encode('utf-8')
             self.is_no_more_events = True
             self.__no_more_event_time = time.time()
         else:
@@ -498,8 +495,6 @@ class ESProcess(threading.Thread):
         except queue.Empty:
             pass
         else:
-            if (sys.version_info > (3, 0)):  # needed for Python 3
-                message = message.decode('utf-8')
             logger.debug('received message from payload: %s' % message)
             if "Ready for events" in message:
                 event_ranges = self.get_event_range_to_payload()
