@@ -84,9 +84,9 @@ def do_use_container(**kwargs):
     copytool = kwargs.get('copytool', False)
     if job:
         # for user jobs, TRF option --containerImage must have been used, ie imagename must be set
-        if job.is_analysis() and job.imagename:
-            use_container = True  # False   WARNING will this change break runcontainer usage?
-            logger.debug('job.is_analysis() and job.imagename -> use_container = True')
+        if job.imagename and job.imagename is not 'NULL':
+            use_container = True
+            logger.debug('job.imagename set -> use_container = True')
         elif not (job.platform or job.alrbuserplatform):
             use_container = False
             logger.debug('not (job.platform or job.alrbuserplatform) -> use_container = False')
